#include "FrameData.hh"
#include "DVector.hh"
#include <stdexcept>

using namespace std;

namespace generator {

  FrameData::InpList  FrameData::mInputList;

  //====================================  Frame data source constructor
  FrameData::FrameData(void) {
      setSample(Interval(0.0));
      mInput = mInputList.end();
  }

  //====================================  Frame data source constructor
   FrameData::FrameData(const char* files, const char* channel) {
      setSample(Interval(0.0));
      mInput = mInputList.end();
      if (files)   setFileList(files);
      if (channel) setChannel(channel);
  }

  //====================================  Frame data source destructor
  FrameData::~FrameData(void) {
  }

  //====================================  Clone a frame data source
  FrameData* 
  FrameData::clone(void) const {
      return new FrameData(*this);
  }

  //====================================  Set the channel ID
  void 
  FrameData::setChannel(const string& value) {
      mChannel = value;
      if (mInput != mInputList.end()) {
	  mInput->refDacc().addChannel(mChannel.c_str());
      }
      DataSource::setParameter("Channel", mChannel);
  }

  //====================================  Set the file list
  void 
  FrameData::setFileList(const string& value) {
      mInput = findInput(value);
      if (mInput == mInputList.end()) mInput = addInput(value);
      mInput->refDacc().setDebug(getDebug());
      if (!mChannel.empty()) {
	  mInput->refDacc().addChannel(mChannel.c_str());
      }
      DataSource::setParameter("Files", value);
  }

  //====================================  Set an arbitrary parameter
  void 
  FrameData::setParameter(const string& name, const string& value) {
      if (getDebug()) cout << "Setting FrameData::" << name << " to " 
			   << value << endl;
      if (name == "Channel")    setChannel(value);
      else if (name == "Files") setFileList(value);
      else                      DataSource::setParameter(name, value);
  }

  FrameData::input::input(const char* fl) {
      if (fl) setFileList(fl);
  }

  FrameData::input::~input(void) {
  }

  void 
  FrameData::input::setFileList(const string& fList) {
      mFileList = fList;
      mIn.addFile(fList.c_str());
  }

  void 
  FrameData::input::getData(const char* chan, const Time& t0, Interval Dt, 
			    int N, float* data) {
      int rc;
      if (mIn.getCurrentTime() < t0) {
	  rc = mIn.seek(t0);
	  if (rc) throw runtime_error("Data not available");
      }

      if (mIn.getFillTime() > t0) throw runtime_error("Data not available");

      Interval Duration = Dt*double(N);
      if (mIn.getCurrentTime() == t0) {
	  rc = mIn.fillData(Duration);
	  if (rc) {
	      cout << "Missing data in channel " << chan << " at " 
		   << t0 << endl;
	      throw runtime_error("Data not available");
	  }
      }

      Time tEnd = t0+Duration;
      int inx = 0;
      for (Time t=t0 ; t<tEnd ; ) {
	  const TSeries* ts=mIn.refData(chan);
	  if (!ts || ts->isEmpty()) {
	      cout << "FrameData: data for channel: " << chan
		   << " not found at " << t << endl;
	      throw runtime_error("Channel not found");
	  } else if (Dt != ts->getTStep()) {
	      cout << "FrameData: Sample rate for channel: " << chan 
		   << " is " << ts->getTStep() << " expected: " << Dt << endl;
	      throw runtime_error("Sample rate mismatch");
	  }
	  int iStart = ts->getBin(t);
	  int len    = ts->getBin(tEnd) - iStart;
	  inx += ts->refDVect()->getData(iStart, len, data+inx);
	  t = ts->getBinT(iStart+len);
	  if (inx < N) {
	      if (mIn.fillData(Dt*double(N-inx))) 
		throw runtime_error("Error getting data");
	  }
      }
  }

  void 
  FrameData::getSeries(const Time& t0, Interval dT, int N, float* data) {
      if (mInput != mInputList.end()) {
	  mInput->refDacc().setDebug(getDebug());
	  mInput->getData(mChannel.c_str(), t0, dT, N, data);
      }
  }

  FrameData::inp_iter 
  FrameData::findInput(const std::string& filelist) {
      for (inp_iter i=mInputList.begin() ; i != mInputList.end() ; ++i) {
	  if (filelist == i->getFileList()) return i;
      }
      return mInputList.end();
  }

  FrameData::inp_iter
  FrameData::addInput(const std::string& fileList) {
      mInputList.push_back(fileList.c_str());
      return --mInputList.end();
  }

}
